from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime


class Purchase(Resource):
    """# Purchase object
     Check out our API Documentation at https://starkbank.com/docs/api#merchant-session
    """

    def __init__(self, amount, card_expiration, card_number, card_security_code, holder_name, funding_type, id=None,
                 holder_email=None, holder_phone=None, installment_count=None, billing_country_code=None,
                 billing_city=None, billing_state_code=None, billing_street_line_1=None, billing_street_line_2=None,
                 billing_zip_code=None, metadata=None, card_ending=None, card_id=None, challenge_mode=None,
                 challenge_url=None, created=None, currency_code=None, end_to_end_id=None, fee=None, network=None,
                 source=None, status=None, tags=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.card_expiration = card_expiration
        self.card_number = card_number
        self.card_security_code = card_security_code
        self.holder_name = holder_name
        self.funding_type = funding_type
        self.holder_email = holder_email
        self.holder_phone = holder_phone
        self.installment_count = installment_count
        self.billing_country_code = billing_country_code
        self.billing_city = billing_city
        self.billing_state_code = billing_state_code
        self.billing_street_line_1 = billing_street_line_1
        self.billing_street_line_2 = billing_street_line_2
        self.billing_zip_code = billing_zip_code
        self.metadata = metadata
        self.card_ending = card_ending
        self.card_id = card_id
        self.challenge_mode = challenge_mode
        self.challenge_url = challenge_url
        self.currency_code = currency_code
        self.end_to_end_id = end_to_end_id
        self.fee = fee
        self.network = network
        self.source = source
        self.status = status
        self.tags = tags
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": Purchase, "name": "Purchase"}




from starkcore.utils.subresource import SubResource


class BoletoPreview(SubResource):

    """# BoletoPreview object
    A BoletoPreview is used to get information from a Boleto payment you received before confirming the payment.
    ## Attributes (return-only):
    - status [string]: current boleto status. ex: "active", "expired" or "inactive"
    - amount [int]: final amount to be paid. ex: 23456 (= R$ 234.56)
    - discount_amount [int]: discount amount to be paid. ex: 23456 (= R$ 234.56)
    - fine_amount [int]: fine amount to be paid. ex: 23456 (= R$ 234.56)
    - interest_amount [int]: interest amount to be paid. ex: 23456 (= R$ 234.56)
    - due [datetime.date]: Boleto due date. ex: 2020-04-30
    - expiration [datetime.date]: Boleto expiration date. ex: 2020-04-30
    - name [string]: beneficiary full name. ex: "Anthony Edward Stark"
    - tax_id [string]: beneficiary tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    - receiver_name [string]: receiver (Sacador Avalista) full name. ex: "Anthony Edward Stark"
    - receiver_tax_id [string]: receiver (Sacador Avalista) tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    - payer_name [string]: payer full name. ex: "Anthony Edward Stark"
    - payer_tax_id [string]: payer tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    - line [string]: Number sequence that identifies the payment. ex: "34191.09008 63571.277308 71444.640008 5 81960000000062"
    - bar_code [string]: Bar code number that identifies the payment. ex: "34195819600000000621090063571277307144464000"
    """

    def __init__(self, status, amount, discount_amount, fine_amount, interest_amount, due, expiration, name, tax_id,
                 receiver_name, receiver_tax_id, payer_name, payer_tax_id, line, bar_code):
        self.status = status
        self.amount = amount
        self.discount_amount = discount_amount
        self.fine_amount = fine_amount
        self.interest_amount = interest_amount
        self.due = due
        self.expiration = expiration
        self.name = name
        self.tax_id = tax_id
        self.receiver_name = receiver_name
        self.receiver_tax_id = receiver_tax_id
        self.payer_name = payer_name
        self.payer_tax_id = payer_tax_id
        self.line = line
        self.bar_code = bar_code


_sub_resource = {"class": BoletoPreview, "name": "BoletoPreview"}



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date, check_datetime_or_date
from ..utils import rest


class CorporateInvoice(Resource):
    """# CorporateInvoice object
    The CorporateInvoice objects created in your Workspace load your Corporate balance when paid.
    When you initialize a CorporateInvoice, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the created object.
    ## Parameters (required):
    - amount [integer]: CorporateInvoice value in cents. ex: 1234 (= R$ 12.34)
    ## Parameters (optional):
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporateInvoice is created. ex: "5656565656565656"
    - name [string, default sub-issuer name]: payer name. ex: "Iron Bank S.A."
    - tax_id [string, default sub-issuer tax ID]: payer tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - brcode [string]: BR Code for the Invoice payment. ex: "00020101021226930014br.gov.bcb.pix2571brcode-h.development.starkbank.com/v2/d7f6546e194d4c64a153e8f79f1c41ac5204000053039865802BR5925Stark Bank S.A. - Institu6009Sao Paulo62070503***63042109"
    - due [datetime.datetime or datetime.date or string]: Invoice due and expiration date in UTC ISO format. ex: "2020-10-28T17:59:26.249976+00:00"
    - link [string]: public Invoice webpage URL. ex: "https://starkbank-card-issuer.development.starkbank.com/invoicelink/d7f6546e194d4c64a153e8f79f1c41ac"
    - status [string]: current CorporateInvoice status. ex: "created", "expired", "overdue", "paid"
    - corporate_transaction_id [string]: ledger transaction ids linked to this CorporateInvoice. ex: "corporate-invoice/5656565656565656"
    - updated [datetime.datetime]: latest update datetime for the CorporateInvoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the CorporateInvoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, tax_id=None, name=None, tags=None, id=None, brcode=None, due=None, link=None, status=None, 
                corporate_transaction_id=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.tax_id = tax_id
        self.name = name
        self.tags = tags
        self.brcode = brcode
        self.due = check_datetime_or_date(due)
        self.link = link
        self.status = status
        self.corporate_transaction_id = corporate_transaction_id
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": CorporateInvoice, "name": "CorporateInvoice"}


def create(invoice, user=None):
    """# Create CorporateInvoice
    Send a CorporateInvoice object for creation at the Stark Bank API
    ## Parameters (required):
    - invoice [CorporateInvoice object]: CorporateInvoice object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateInvoice object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=invoice, user=user)


def query(limit=None, after=None, before=None, status=None, tags=None, user=None):
    """# Retrieve CorporateInvoices
    Receive a generator of CorporateInvoice objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "expired", "overdue", "paid"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - generator of CorporateInvoice objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        status=status,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, user=None):
    """# Retrieve CorporateInvoices
    Receive a list of up to 100 CorporateInvoice objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "expired", "overdue", "paid"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - list of CorporateInvoice objects with updated attributes
    - cursor to retrieve the next page of CorporateInvoice objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        status=status,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime, check_datetime_or_date

class MerchantCard(Resource):
    """# MerchantCard object
    Check out our API Documentation at https://starkbank.com/docs/api#merchant-card
    """

    def __init__(self, id=None, ending=None , funding_type=None, holder_name=None, network=None, status=None, tags=None,
                 expiration=None, created=None, updated=None):
        Resource.__init__(self, id=id)
        self.ending = ending
        self.funding_type = funding_type
        self.holder_name = holder_name
        self.network = network
        self.status = status
        self.tags = tags
        self.expiration = check_datetime_or_date(expiration)
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": MerchantCard, "name": "MerchantCard"}


def get(id, user=None):
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, tags=None, ids=None, user=None):
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, ids=None, user=None):
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )



import base64
from starkcore.utils.checks import check_datetime
from ..utils import rest
from starkcore.utils.resource import Resource


class Workspace(Resource):
    """# Workspace object
    Workspaces are bank accounts. They have independent balances, statements, operations and permissions.
    The only property that is shared between your workspaces is that they are linked to your organization,
    which carries your basic informations, such as tax ID, name, etc..
    ## Parameters (required):
    - username [string]: Simplified name to define the workspace URL. This name must be unique across all Stark Bank Workspaces. ex: "starkbankworkspace"
    - name [string]: Full name that identifies the Workspace. This name will appear when people access the Workspace on our platform, for example. ex: "Stark Bank Workspace"
    ## Parameters (optional):
    - allowed_tax_ids [list of strings, default None]: list of tax IDs that will be allowed to send Deposits to this Workspace. If empty, all are allowed. ex: ["012.345.678-90", "20.018.183/0001-80"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the workspace is created. ex: "5656565656565656"
    - status [string]: current Workspace status. Options: "active", "closed", "frozen" or "blocked"
    - organization_id [string]: unique organization id returned when the organization is created. ex: "5656565656565656"
    - picture_url [string]: public workspace image (png) URL. ex: "https://storage.googleapis.com/api-ms-workspace-sbx.appspot.com/pictures/workspace/6284441752174592.png?20230208220551"
    - created [datetime.datetime]: creation datetime of the workspace. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, username, name, allowed_tax_ids=None, id=None, status=None, organization_id=None, picture_url=None,
                 created=None):
        Resource.__init__(self, id=id)

        self.username = username
        self.name = name
        self.allowed_tax_ids = allowed_tax_ids
        self.status = status
        self.organization_id = organization_id
        self.picture_url = picture_url
        self.created = check_datetime(created)


_resource = {"class": Workspace, "name": "Workspace"}


def create(username, name, allowed_tax_ids=None, user=None):
    """# Create Workspace
    Send a Workspace for creation in the Stark Bank API
    ## Parameters (required):
    - username [string]: Simplified name to define the workspace URL. This name must be unique across all Stark Bank Workspaces. ex: "starkbankworkspace"
    - name [string]: Full name that identifies the Workspace. This name will appear when people access the Workspace on our platform, for example. ex: "Stark Bank Workspace"
    ## Parameters (optional):
    - allowed_tax_ids [list of strings, default None]: list of tax IDs that will be allowed to send Deposits to this Workspace. If empty, all are allowed. ex: ["012.345.678-90", "20.018.183/0001-80"]
    - user [Organization object, default None]: Organization object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Workspace object with updated attributes
    """
    workspace = Workspace(username=username, name=name, allowed_tax_ids=allowed_tax_ids)
    return rest.post_single(resource=_resource, entity=workspace, user=user)


def get(id, user=None):
    """# Retrieve a specific Workspace
    Receive a single Workspace object previously created in the Stark Bank API by passing its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Workspace object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, username=None, ids=None, user=None):
    """# Retrieve Workspaces
    Receive a generator of Workspace objects previously created in the Stark Bank API.
    If no filters are passed and the user is an Organization, all of the Organization Workspaces
    will be retrieved.
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - username [string, default None]: query by the simplified name that defines the workspace URL. This name is always unique across all Stark Bank Workspaces. ex: "starkbankworkspace"
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Workspace objects with updated attributes
    """
    return rest.get_stream(resource=_resource, limit=limit, username=username, ids=ids, user=user)


def update(id, username=None, name=None, allowed_tax_ids=None, status=None, picture=None, picture_type=None, user=None):
    """# Update Workspace entity
    Update a Workspace by passing its ID.
    ## Parameters (required):
    - id [string]: Workspace ID. ex: '5656565656565656'
    ## Parameters (conditionally required):
    - picture_type [string]: picture MIME type. This parameter will be required if the picture parameter is informed ex: "image/png" or "image/jpeg"
    ## Parameters (optional):
    - username [string, default None]: Simplified name to define the workspace URL. This name must be unique across all Stark Bank Workspaces. ex: "starkbank-workspace"
    - name [string, default None]: Full name that identifies the Workspace. This name will appear when people access the Workspace on our platform, for example. ex: "Stark Bank Workspace"
    - allowed_tax_ids [list of strings, default None]: list of tax IDs that will be allowed to send Deposits to this Workspace. If empty, all are allowed. ex: ["012.345.678-90", "20.018.183/0001-80"]
    - status [string, default None]: current Workspace status. Options: "active" or "blocked"
    - picture [bytes, default None]: Binary buffer of the picture. ex: open("/path/to/file.png", "rb").read()
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - target Workspace with updated attributes
    """
    payload = {
        "allowedTaxIds": allowed_tax_ids,
        "status": status,
        "name": name,
        "username": username,
    }

    if picture:
        payload["picture"] = "data:{picture_type};base64,{picture}".format(
            picture_type=picture_type,
            picture=base64.b64encode(picture).decode('utf-8')
        )

    return rest.patch_id(resource=_resource, id=id, user=user, username=username, name=name, payload=payload)


def page(cursor=None, limit=None, username=None, ids=None, user=None):
    """# Retrieve paged Workspaces
    Receive a list of up to 100 Workspace objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - username [string, default None]: query by the simplified name that defines the workspace URL. This name is always unique across all Stark Bank Workspaces. ex: "starkbankworkspace"
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Workspace objects with updated attributes
    - cursor to retrieve the next page of Workspace objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        username=username,
        ids=ids,
        user=user,
    )



from starkcore.utils.subresource import SubResource


class BrcodePreview(SubResource):

    """# BrcodePreview object
    A BrcodePreview is used to get information from a BR Code you received before confirming the payment.
    ## Attributes (return-only):
    - status [string]: Payment status. ex: "created", "paid", "canceled" or "expired"
    - name [string]: Payment receiver name. ex: "Tony Stark"
    - tax_id [string]: Payment receiver tax ID. ex: "012.345.678-90"
    - bank_code [string]: Payment receiver bank code. ex: "20018183"
    - account_type [string]: Payment receiver account type. ex: "checking"
    - allow_change [bool]: If True, the payment is able to receive amounts that are different from the nominal one. ex: True or False
    - amount [integer]: Value in cents that this payment is expecting to receive. If 0, any value is accepted. ex: 123 (= R$1,23)
    - nominal_amount [integer]: Original value in cents that this payment was expecting to receive without the discounts, fines, etc.. If 0, any value is accepted. ex: 123 (= R$1,23)
    - interest_amount [integer]: Current interest value in cents that this payment is charging. If 0, any value is accepted. ex: 123 (= R$1,23)
    - fine_amount [integer]: Current fine value in cents that this payment is charging. ex: 123 (= R$1,23)
    - reduction_amount [integer]: Current value reduction value in cents that this payment is expecting. ex: 123 (= R$1,23)
    - discount_amount [integer]: Current discount value in cents that this payment is expecting. ex: 123 (= R$1,23)
    - reconciliation_id [string]: Reconciliation ID linked to this payment. ex: "txId", "payment-123"
    - description [string]: Payment description informed by the receiver. ex: "Payment for service #1234"
    """

    def __init__(self, status, name, tax_id, bank_code, account_type, allow_change, amount, nominal_amount,
                 interest_amount, fine_amount, reduction_amount, discount_amount, reconciliation_id, description):
        self.status = status
        self.name = name
        self.tax_id = tax_id
        self.bank_code = bank_code
        self.account_type = account_type
        self.allow_change = allow_change
        self.amount = amount
        self.nominal_amount = nominal_amount
        self.interest_amount = interest_amount
        self.fine_amount = fine_amount
        self.reduction_amount = reduction_amount
        self.discount_amount = discount_amount
        self.reconciliation_id = reconciliation_id
        self.description = description


_sub_resource = {"class": BrcodePreview, "name": "BrcodePreview"}



from ..utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..transfer.__transfer import Transfer
from ..transaction.__transaction import Transaction
from ..brcodepayment.__brcodepayment import BrcodePayment
from ..boletopayment.__boletopayment import BoletoPayment
from ..utilitypayment.__utilitypayment import UtilityPayment
from ..darfpayment.__darfpayment import DarfPayment
from ..taxpayment.__taxpayment import TaxPayment
from ..transfer.__transfer import _resource as _transfer_resource
from ..transaction.__transaction import _resource as _transaction_resource
from ..boletopayment.__boletopayment import _resource as _boleto_payment_resource
from ..brcodepayment.__brcodepayment import _resource as _brcode_payment_resource
from ..utilitypayment.__utilitypayment import _resource as _utility_payment_resource
from ..darfpayment.__darfpayment import _resource as _darf_payment_resource
from ..taxpayment.__taxpayment import _resource as _tax_payment_resource
class PaymentRequest(Resource):
    """# PaymentRequest object
    A PaymentRequest is an indirect request to access a specific cash-out service
    (such as Transfers, BoletoPayments, etc.) which goes through the cost center
    approval flow on our website. To emit a PaymentRequest, you must direct it to
    a specific cost center by its ID, which can be retrieved on our website at the
    cost center page.
    ## Parameters (required):
    - center_id [string]: target cost center ID. ex: "5656565656565656"
    - payment [Transfer, BoletoPayment, UtilityPayment, BrcodePayment, Transaction, DarfPayment, TaxPayment or dictionary]: payment entity that should be approved and executed.
    ## Parameters (conditionally required):
    - type [string]: payment type, inferred from the payment parameter if it is not a dictionary. ex: "transfer", "boleto-payment"
    ## Parameters (optional):
    - due [datetime.date or string, default today]: Payment target date in ISO format. ex: 2020-04-30
    - tags [list of strings]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when a PaymentRequest is created. ex: "5656565656565656"
    - amount [integer]: PaymentRequest amount. ex: 100000 = R$1.000,00
    - description [string]: payment request description. ex: "Tony Stark's Suit"
    - status [string]: current PaymentRequest status. ex: "pending" or "approved"
    - actions [list of dictionaries]: list of actions that are affecting this PaymentRequest. ex: [{"type": "member", "id": "56565656565656, "action": "requested"}]
    - updated [datetime.datetime]: latest update datetime for the PaymentRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the PaymentRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, center_id, payment, type=None, due=None, tags=None, id=None, amount=None,
                 description=None, status=None, actions=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.center_id = center_id
        self.due = due
        self.tags = tags
        self.amount = amount
        self.description = description
        self.status = status
        self.actions = actions
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)

        self.payment, self.type = _parse_payment(payment=payment, type=type)


def _parse_payment(payment, type):
    if isinstance(payment, dict):
        try:
            return from_api_json(*({
                "transfer": _transfer_resource,
                "transaction": _transaction_resource,
                "boleto-payment": _boleto_payment_resource,
                "brcode-payment": _brcode_payment_resource,
                "utility-payment": _utility_payment_resource,
                "darf-payment": _darf_payment_resource,
                "tax-payment": _tax_payment_resource,
            }[type], payment)), type
        except KeyError:
            return payment, type

    if type:
        return payment, type

    if isinstance(payment, Transfer):
        return payment, "transfer"
    if isinstance(payment, Transaction):
        return payment, "transaction"
    if isinstance(payment, BrcodePayment):
        return payment, "brcode-payment"
    if isinstance(payment, BoletoPayment):
        return payment, "boleto-payment"
    if isinstance(payment, UtilityPayment):
        return payment, "utility-payment"
    if isinstance(payment, DarfPayment):
        return payment, "darf-payment"
    if isinstance(payment, TaxPayment):
        return payment, "tax-payment"

    raise Exception(
        "payment must be either "
        "a dictionary"
        ", a starkbank.Transfer"
        ", a starkbank.Transaction"
        ", a starkbank.BrcodePayment"
        ", a starkbank.BoletoPayment"
        ", a starkbank.UtilityPayment"
        ", a starkbank.TaxPayment"
        " or a starkbank.DarfPayment"
        ", but not a {}".format(type(payment))
    )


_resource = {"class": PaymentRequest, "name": "PaymentRequest"}


def create(requests, user=None):
    """# Create PaymentRequests
    Send a list of PaymentRequest objects for creation in the Stark Bank API
    ## Parameters (required):
    - requests [list of PaymentRequest objects]: list of PaymentRequest objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of PaymentRequest objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=requests, user=user)


def query(center_id, limit=None, after=None, before=None, sort=None, status=None, type=None, tags=None, ids=None, user=None):
    """# Retrieve PaymentRequests
    Receive a generator of PaymentRequest objects previously created by this user in the Stark Bank API
    ## Parameters (required):
    - center_id [string]: target cost center ID. ex: "5656565656565656"
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - sort [string, default "-created"]: sort order considered in response. Valid options are "-created" or "-due".
    - status [string, default None]: filter for status of retrieved objects. ex: "success" or "failed"
    - type [string, default None]: payment type, inferred from the payment parameter if it is not a dictionary. ex: "transfer", "boleto-payment"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of PaymentRequest objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        center_id=center_id,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        sort=sort,
        status=status,
        type=type,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(center_id, cursor=None, limit=None, after=None, before=None, sort=None, status=None, type=None, tags=None, ids=None, user=None):
    """# Retrieve paged PaymentRequests
    Receive a list of up to 100 PaymentRequest objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (required):
    - center_id [string]: target cost center ID. ex: "5656565656565656"
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - sort [string, default "-created"]: sort order considered in response. Valid options are "-created" or "-due".
    - status [string, default None]: filter for status of retrieved objects. ex: "success" or "failed"
    - type [string, default None]: payment type, inferred from the payment parameter if it is not a dictionary. ex: "transfer", "boleto-payment"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of PaymentRequest objects with updated attributes
    - cursor to retrieve the next page of PaymentRequest objects
    """
    return rest.get_page(
        resource=_resource,
        center_id=center_id,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        sort=sort,
        status=status,
        type=type,
        tags=tags,
        ids=ids,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime


class Balance(Resource):
    """# Balance object
    The Balance object displays the current balance of the workspace,
    which is the result of the sum of all transactions within this
    workspace. The balance is never generated by the user, but it
    can be retrieved to see the available information.
    ## Attributes (return-only):
    - id [string]: unique id returned when Balance is created. ex: "5656565656565656"
    - amount [integer]: current balance amount of the workspace in cents. ex: 200 (= R$ 2.00)
    - currency [string]: currency of the current workspace. Expect others to be added eventually. ex: "BRL"
    - updated [datetime.datetime]: latest update datetime for the balance. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, currency, updated, id):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.currency = currency
        self.updated = check_datetime(updated)


_resource = {"class": Balance, "name": "Balance"}


def get(user=None):
    """# Retrieve the Balance object
    Receive the Balance object linked to your workspace in the Stark Bank API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Balance object with updated attributes
    """
    return next(rest.get_stream(resource=_resource, user=user))



# coding: utf-8
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_datetime, check_date, check_timedelta
from ..utils import rest
from .rule.__rule import _sub_resource as _rule_resource, Rule


class DynamicBrcode(Resource):
    """# DynamicBrcode object
    Check out our API Documentation at https://starkbank.com/docs/api#dynamic-brcode
    """

    def __init__(self, amount, expiration=None, tags=None, display_description=None, rules=None, id=None, uuid=None,
                 picture_url=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.expiration = check_timedelta(expiration)
        self.display_description = display_description
        self.rules = _parse_rules(rules)
        self.tags = tags
        self.uuid = uuid
        self.picture_url = picture_url
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)

_resource = {"class": DynamicBrcode, "name": "DynamicBrcode"}


def _parse_rules(rules):
    if rules is None:
        return None
    parsed_rules = []
    for rule in rules:
        if isinstance(rule, Rule):
            parsed_rules.append(rule)
            continue
        parsed_rules.append(from_api_json(_rule_resource, rule))
    return parsed_rules


def create(brcodes, user=None):
    """# Create DynamicBrcodes
    Send a list of DynamicBrcode objects for creation in the Stark Bank API
    ## Parameters (required):
    - brcodes [list of DynamicBrcode objects]: list of DynamicBrcode objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of DynamicBrcode objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=brcodes, user=user)


def get(uuid, user=None):
    """# Retrieve a specific DynamicBrcode
    Receive a single DynamicBrcode object previously created in the Stark Bank API by its uuid
    ## Parameters (required):
    - uuid [string]: object's unique uuid. ex: "901e71f2447c43c886f58366a5432c4b"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - DynamicBrcode object with updated attributes
    """
    return rest.get_id(resource=_resource, id=uuid, user=user)


def query(limit=None, after=None, before=None, tags=None, uuids=None, user=None):
    """# Retrieve DynamicBrcodes
    Receive a generator of DynamicBrcode objects previously created by this user in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - uuids [list of strings, default None]: list of uuids to filter retrieved objects. ex: ["901e71f2447c43c886f58366a5432c4b", "4e2eab725ddd495f9c98ffd97440702d"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of DynamicBrcode objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        uuids=uuids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, tags=None, uuids=None, user=None):
    """# Retrieve paged DynamicBrcodes
    Receive a list of up to 100 DynamicBrcode objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your brcodes.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - uuids [list of strings, default None]: list of uuids to filter retrieved objects. ex: ["901e71f2447c43c886f58366a5432c4b", "4e2eab725ddd495f9c98ffd97440702d"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of DynamicBrcode objects with updated attributes
    - cursor to retrieve the next page of DynamicBrcode objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        uuids=uuids,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class TaxPayment(Resource):
    """# TaxPayment object
    When you initialize a TaxPayment, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (conditionally required):
    - line [string, default None]: Number sequence that describes the payment. Either 'line' or 'bar_code' parameters are required. If both are sent, they must match. ex: "85800000003 0 28960328203 1 56072020190 5 22109674804 0"
    - bar_code [string, default None]: Bar code number that describes the payment. Either 'line' or 'barCode' parameters are required. If both are sent, they must match. ex: "83660000001084301380074119002551100010601813"
    ## Parameters (required):
    - description [string]: Text to be displayed in your statement (min. 10 characters). ex: "payment ABC"
    ## Parameters (optional):
    - scheduled [datetime.date or string, default today]: payment scheduled date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when payment is created. ex: "5656565656565656"
    - type [string]: tax type. ex: "das"
    - status [string]: current payment status. ex: "success" or "failed"
    - amount [int]: amount automatically calculated from line or bar_code. ex: 23456 (= R$ 234.56)
    - fee [integer]: fee charged when tax payment is created. ex: 200 (= R$ 2.00)
    - transaction_ids [list of strings]: ledger transaction ids linked to this TaxPayment. ex: ["19827356981273"]
    - updated [datetime.datetime]: latest update datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, description, line=None, bar_code=None, tags=None, scheduled=None, id=None, type=None, amount=None,
                 fee=None, status=None, transaction_ids=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.line = line
        self.bar_code = bar_code
        self.description = description
        self.tags = tags
        self.scheduled = check_date(scheduled)
        self.status = status
        self.amount = amount
        self.fee = fee
        self.type = type
        self.transaction_ids = transaction_ids
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": TaxPayment, "name": "TaxPayment"}


def create(payments, user=None):
    """# Create TaxPayments
    Send a list of TaxPayment objects for creation in the Stark Bank API
    ## Parameters (required):
    - payments [list of TaxPayment objects]: list of TaxPayment objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of TaxPayment objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=payments, user=user)


def get(id, user=None):
    """# Retrieve a specific TaxPayment
    Receive a single TaxPayment object previously created by the Stark Bank API by passing its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - TaxPayment object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific TaxPayment pdf file
    Receive a single TaxPayment pdf file generated in the Stark Bank API by passing its id.
    Only valid for tax payments with "success" status.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - TaxPayment pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def query(limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve TaxPayments
    Receive a generator of TaxPayment objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Project object, default None]: Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of TaxPayment objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve paged TaxPayments
    Receive a list of up to 100 TaxPayment objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of TaxPayment objects with updated attributes
    - cursor to retrieve the next page of TaxPayment objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def delete(id, user=None):
    """# Delete a TaxPayment entity
    Delete a TaxPayment entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: TaxPayment unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deleted TaxPayment object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime, check_datetime_or_date


class MerchantInstallment(Resource):
    """# MerchantInstallment object
    Check out our API Documentation at https://starkbank.com/docs/api#merchant-installment
    """

    def __init__(self, id=None, amount=None, created=None, due=None, fee=None, funding_type=None, network=None,
                 purchase_id=None, status=None, tags=None, transaction_ids=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.due = check_datetime_or_date(due)
        self.fee = fee
        self.funding_type = funding_type
        self.network = network
        self.purchase_id = purchase_id
        self.status = status
        self.tags = tags
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": MerchantInstallment, "name": "MerchantInstallment"}


def get(id, user=None):
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, tags=None, ids=None, user=None, purchase_ids=None):
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
        purchase_ids=purchase_ids,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, ids=None, user=None, purchase_ids=None):
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
        purchase_ids=purchase_ids,
    )




from ...utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..__boletoholmes import _resource as _holmes_resource


class Log(Resource):
    """# boletoholmes.Log object
    Every time a BoletoHolmes entity is modified, a corresponding boletoholmes.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the BoletoHolmes.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - holmes [BoletoHolmes]: BoletoHolmes entity to which the log refers to.
    - type [string]: type of the BoletoHolmes event which triggered the log creation. ex: "solving" or "solved"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, created, updated, type, holmes):
        Resource.__init__(self, id=id)

        self.created = check_datetime(created)
        self.updated = check_datetime(updated)
        self.type = type
        self.holmes = from_api_json(_holmes_resource, holmes)


_resource = {"class": Log, "name": "BoletoHolmesLog"}


def get(id, user=None):
    """# Retrieve a specific boletoHolmes.Log
    Receive a single boletoHolmes.Log object previously created by the Stark Bank API by passing its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - boletoholmes.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, types=None, holmes_ids=None, user=None):
    """# Retrieve boletoholmes.Logs
    Receive a generator of boletoholmes.Log objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by event type. ex: ["solving", "solved"]
    - holmes_ids [list of strings, default None]: list of BoletoHolmes ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of boletoholmes.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        holmes_ids=holmes_ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, types=None, holmes_ids=None, user=None):
    """# Retrieve paged boletoholmes.Logs
    Receive a list of up to 100 boletoholmes.Log objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by event type. ex: ["solving", "solved"]
    - holmes_ids [list of strings, default None]: list of boletoholmes.Log ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of boletoholmes.Log objects with updated attributes
    - cursor to retrieve the next page of boletoholmes.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        holmes_ids=holmes_ids,
        user=user,
    )



from .__merchantsession import create, get, query, page, purchase
from .log.__log import Log
from . import log
from .__purchase import Purchase
from .allowedinstallment.__allowedinstallment import AllowedInstallment



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ...utils import rest
from ..__corporatepurchase import _resource as _corporate_purchase_resource
from starkcore.error import Error


class Log(Resource):
    """# corporatepurchase.Log object
    Every time a CorporatePurchase entity is updated, a corresponding corporatepurchase.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the CorporatePurchase.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - purchase [CorporatePurchase]: CorporatePurchase entity to which the log refers to.
    - description [string]: purchase descriptions. ex: "my_description"
    - corporate_transaction_id [string]: transaction ID related to the CorporateCard.
    - errors [list of StarkCore.Error]: list of errors linked to this CorporatePurchase event.
    - type [string]: type of the CorporatePurchase event which triggered the log creation. ex: "approved", "canceled", "confirmed", "denied", "reversed", "voided"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, purchase, description, corporate_transaction_id, errors, type, created):
        Resource.__init__(self, id=id)

        self.purchase = from_api_json(_corporate_purchase_resource, purchase)
        self.description = description
        self.corporate_transaction_id = corporate_transaction_id
        self.errors = _parse_errors(errors)
        self.type = type
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "CorporatePurchaseLog"}


def get(id, user=None):
    """# Retrieve a specific corporatepurchase.Log
    Receive a single corporatepurchase.Log object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - corporatepurchase.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, types=None, purchase_ids=None, user=None):
    """# Retrieve corporatepurchase.Log
    Receive a generator of corporatepurchase.Log objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["approved", "canceled", "confirmed", "denied", "reversed", "voided"]
    - purchase_ids [list of strings, default None]: list of Purchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of CorporatePurchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of corporatepurchase.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        purchase_ids=purchase_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, purchase_ids=None, user=None):
    """# Retrieve paged corporatepurchase.Log
    Receive a list of up to 100 corporatepurchase.Log objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["approved", "canceled", "confirmed", "denied", "reversed", "voided"]
    - purchase_ids [list of strings, default None]: list of Purchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of CorporatePurchase ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of corporatepurchase.Log objects with updated attributes
    - cursor to retrieve the next page of corporatepurchase.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        purchase_ids=purchase_ids,
        user=user,
    )

def _parse_errors(errors):
    parsed_errors = []
    for error in errors:
        if isinstance(error, Error):
            parsed_errors.append(error)
            continue
        parsed_errors.append(Error(code=error['code'], message=error['message']))
    return parsed_errors



from starkcore.utils.resource import Resource
from ..utils import rest


class CorporateBalance(Resource):
    """# CorporateBalance object
    The CorporateBalance object displays the current corporate balance of the Workspace,
    which is the result of the sum of all transactions within this
    Workspace. The balance is never generated by the user, but it
    can be retrieved to see the available information.
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporateBalance is created. ex: "5656565656565656"
    - amount [integer]: current balance amount of the Workspace in cents. ex: 200 (= R$ 2.00)
    - limit [integer]: The maximum negative balance allowed by the user. ex: 10000 (= R$ 100.00)
    - max_limit [integer]: The maximum negative balance allowed by StarkBank. ex: 100000 (= R$ 1000.00)
    - currency [string]: currency of the current Workspace. Expect others to be added eventually. ex: "BRL"
    - updated [datetime.datetime]: latest update datetime for the CorporateBalance. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, amount, limit, max_limit, currency, updated):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.limit = limit
        self.max_limit = max_limit
        self.currency = currency
        self.updated = updated


_resource = {"class": CorporateBalance, "name": "CorporateBalance"}


def get(user=None):
    """# Retrieve the CorporateBalance object
    Receive the CorporateBalance object linked to your Workspace in the Stark Bank API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateBalance object with updated attributes
    """
    return next(rest.get_stream(resource=_resource, user=user))



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class SplitReceiver(Resource):
    """# SplitReceiver object
    When you initialize a SplitReceiver, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - name [string]: receiver full name. ex: "Anthony Edward Stark"
    - tax_id [string]: receiver account tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - bank_code [string]: code of the receiver bank institution in Brazil. If an ISPB (8 digits) is informed, a PIX splitReceiver will be created, else a TED will be issued. ex: "20018183" or "341"
    - branch_code [string]: receiver bank account branch. Use '-' in case there is a verifier digit. ex: "1357-9"
    - account_number [string]: receiver bank account number. Use '-' before the verifier digit. ex: "876543-2"
    - account_type [string]: Receiver bank account type. This parameter only has effect on Pix SplitReceivers. ex: "checking", "savings", "salary" or "payment"
    ## Parameters (optional):
    - tags [list of strings, default []]: list of strings for reference when searching for receivers. ex: ["seller/123456"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the splitReceiver is created. ex: "5656565656565656"
    - status [string]: current splitReceiver status. ex: "success" or "failed"
    - created [datetime.datetime]: creation datetime for the splitReceiver. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the splitReceiver. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, tax_id, bank_code, branch_code, account_number, account_type,
                 tags=None, id=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.tax_id = tax_id
        self.bank_code = bank_code
        self.branch_code = branch_code
        self.account_number = account_number
        self.account_type = account_type
        self.tags = tags
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": SplitReceiver, "name": "SplitReceiver"}


def create(splitReceivers, user=None):
    """# Create SplitReceivers
    Send a list of SplitReceiver objects for creation in the Stark Bank API
    ## Parameters (required):
    - splitReceivers [list of SplitReceiver objects]: list of SplitReceiver objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of SplitReceiver objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=splitReceivers, user=user)


def get(id, user=None):
    """# Retrieve a specific SplitReceiver
    Receive a single SplitReceiver object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - SplitReceiver object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, transaction_ids=None, status=None, tax_id=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve SplitReceivers
    Receive a generator of SplitReceiver objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - transaction_ids [list of strings, default None]: list of transaction IDs linked to the desired splitReceivers. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success" or "failed"
    - tax_id [string, default None]: filter for splitReceivers sent to the specified tax ID. ex: "012.345.678-90"
    - sort [string, default "-created"]: sort order considered in response. Valid options are "created", "-created", "updated" or "-updated".
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of SplitReceiver objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        transaction_ids=transaction_ids,
        status=status,
        tax_id=tax_id,
        sort=sort,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve paged SplitReceivers
    Receive a list of up to 100 SplitReceiver objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "success" or "failed"
    - sort [string, default "-created"]: sort order considered in response. Valid options are "created", "-created", "updated" or "-updated".
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of SplitReceiver objects with updated attributes
    - cursor to retrieve the next page of SplitReceiver objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        sort=sort,
        tags=tags,
        ids=ids,
        user=user,
    )



from ..utils import rest
from .rule.__rule import Rule
from .rule.__rule import _sub_resource as _rule_resource
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date, check_datetime_or_date


class Transfer(Resource):
    """# Transfer object
    When you initialize a Transfer, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - amount [integer]: amount in cents to be transferred. ex: 1234 (= R$ 12.34)
    - name [string]: receiver full name. ex: "Anthony Edward Stark"
    - tax_id [string]: receiver tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - bank_code [string]: code of the receiver bank institution in Brazil. If an ISPB (8 digits) is informed, a PIX transfer will be created, else a TED will be issued. ex: "20018183" or "341"
    - branch_code [string]: receiver bank account branch. Use '-' in case there is a verifier digit. ex: "1357-9"
    - account_number [string]: receiver bank account number. Use '-' before the verifier digit. ex: "876543-2"
    - account_type [string]: Receiver bank account type. This parameter only has effect on Pix Transfers. ex: "checking", "savings", "salary" or "payment"
    ## Parameters (optional):
    - external_id [string, default None]: url safe string that must be unique among all your transfers. Duplicated external_ids will cause failures. By default, this parameter will block any transfer that repeats amount and receiver information on the same date. ex: "my-internal-id-123456"
    - scheduled [datetime.date, datetime.datetime or string, default now]: date or datetime when the transfer will be processed. May be pushed to next business day if necessary. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - description [string, default None]: optional description to override default description to be shown in the bank statement. ex: "Payment for service #1234"
    - tags [list of strings, default []]: list of strings for reference when searching for transfers. ex: ["employees", "monthly"]
    - rules [list of Transfer.Rules, default []]: list of Transfer.Rule objects for modifying transfer behavior. ex: [Transfer.Rule(key="resendingLimit", value=5)]
    ## Attributes (return-only):
    - id [string]: unique id returned when the transfer is created. ex: "5656565656565656"
    - fee [integer]: fee charged when the Transfer is processed. ex: 200 (= R$ 2.00)
    - status [string]: current transfer status. ex: "success" or "failed"
    - transaction_ids [list of strings]: ledger Transaction IDs linked to this Transfer (if there are two, the second is the chargeback). ex: ["19827356981273"]
    - metadata [dictionary]: dictionary object used to store additional information about the Transfer object.
    - created [datetime.datetime]: creation datetime for the transfer. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the transfer. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, name, tax_id, bank_code, branch_code, account_number, account_type,
                 external_id=None, scheduled=None, description=None, transaction_ids=None, metadata=None, 
                 fee=None, tags=None, rules=None, status=None, id=None, created=None, updated=None
                ):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.name = name
        self.tax_id = tax_id
        self.bank_code = bank_code
        self.branch_code = branch_code
        self.account_number = account_number
        self.account_type = account_type
        self.external_id = external_id
        self.scheduled = check_datetime_or_date(scheduled)
        self.description = description
        self.tags = tags
        self.rules = _parse_rules(rules)
        self.fee = fee
        self.status = status
        self.transaction_ids = transaction_ids
        self.metadata = metadata
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": Transfer, "name": "Transfer"}


def _parse_rules(rules):
    if rules is None:
        return None
    parsed_rules = []
    for rule in rules:
        if isinstance(rule, Rule):
            parsed_rules.append(rule)
            continue
        parsed_rules.append(from_api_json(_rule_resource, rule))
    return parsed_rules


def create(transfers, user=None):
    """# Create Transfers
    Send a list of Transfer objects for creation in the Stark Bank API
    ## Parameters (required):
    - transfers [list of Transfer objects]: list of Transfer objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Transfer objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=transfers, user=user)


def get(id, user=None):
    """# Retrieve a specific Transfer
    Receive a single Transfer object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Transfer object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def delete(id, user=None):
    """# Delete a Transfer entity
    Delete a Transfer entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deleted Transfer object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific Transfer pdf file
    Receive a single Transfer pdf receipt file generated in the Stark Bank API by its id.
    Only valid for transfers with "processing" and "success" status.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Transfer pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def query(limit=None, after=None, before=None, transaction_ids=None, status=None, tax_id=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve Transfers
    Receive a generator of Transfer objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - transaction_ids [list of strings, default None]: list of transaction IDs linked to the desired transfers. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success" or "failed"
    - tax_id [string, default None]: filter for transfers sent to the specified tax ID. ex: "012.345.678-90"
    - sort [string, default "-created"]: sort order considered in response. Valid options are "created", "-created", "updated" or "-updated".
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Transfer objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        transaction_ids=transaction_ids,
        status=status,
        tax_id=tax_id,
        sort=sort,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, transaction_ids=None, status=None, tax_id=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve paged Transfers
    Receive a list of up to 100 Transfer objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - transaction_ids [list of strings, default None]: list of transaction IDs linked to the desired transfers. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success" or "failed"
    - tax_id [string, default None]: filter for transfers sent to the specified tax ID. ex: "012.345.678-90"
    - sort [string, default "-created"]: sort order considered in response. Valid options are "created", "-created", "updated" or "-updated".
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Transfer objects with updated attributes
    - cursor to retrieve the next page of Transfer objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        transaction_ids=transaction_ids,
        status=status,
        tax_id=tax_id,
        sort=sort,
        tags=tags,
        ids=ids,
        user=user,
    )



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest


class CorporateTransaction(Resource):
    """# CorporateTransaction object
    The CorporateTransaction object created in your Workspace to represent each balance shift.
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporateTransaction is created. ex: "5656565656565656"
    - amount [integer]: CorporateTransaction value in cents. ex: 1234 (= R$ 12.34)
    - balance [integer]: balance amount of the Workspace at the instant of the Transaction in cents. ex: 200 (= R$ 2.00)
    - description [string]: CorporateTransaction description. ex: "Buying food"
    - source [string]: source of the transaction. ex: "corporate-purchase/5656565656565656"
    - tags [string]: list of strings inherited from the source resource. ex: ["tony", "stark"]
    - created [datetime.datetime]: creation datetime for the CorporateTransaction. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, amount, balance, description, source, tags, created):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.balance = balance
        self.description = description
        self.source = source
        self.tags = tags
        self.created = created


_resource = {"class": CorporateTransaction, "name": "CorporateTransaction"}


def get(id, user=None):
    """# Retrieve a specific CorporateTransaction
    Receive a single CorporateTransaction object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateTransaction object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(source=None, tags=None, external_ids=None, after=None, before=None,
          ids=None, limit=None, user=None):
    """# Retrieve CorporateTransaction
    Receive a generator of CorporateTransaction objects previously created in the Stark Bank API
    ## Parameters (optional):
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - ids [list of strings, default None]: purchase IDs
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - generator of CorporateTransaction objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        source=source,
        tags=tags,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        limit=limit,
        user=user,
    )


def page(source=None, tags=None, external_ids=None, after=None, before=None,
         ids=None, limit=None, cursor=None, user=None):
    """# Retrieve paged CorporateTransaction
    Receive a list of up to 100 CorporateTransaction objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - ids [list of strings, default None]: purchase IDs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - list of CorporateTransaction objects with updated attributes
    - cursor to retrieve the next page of CorporatePurchase objects
    """
    return rest.get_page(
        resource=_resource,
        source=source,
        tags=tags,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        limit=limit,
        cursor=cursor,
        user=user,
    )



from starkcore.utils.api import endpoint, last_name, from_api_json, api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime
from ..utils import rest
from ..corporaterule import parse_rules


class CorporateCard(Resource):
    """# CorporateCard object
    The CorporateCard object displays the information of the cards created in your Workspace.
    Sensitive information will only be returned when the "expand" parameter is used, to avoid security concerns.
    When you initialize a CorporateCard, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the created object.
    ## Parameters (required):
    - holder_id [string]: card holder unique id. ex: "5656565656565656"
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporateCard is created. ex: "5656565656565656"
    - holder_name [string]: card holder name. ex: "Tony Stark"
    - display_name [string]: card displayed name. ex: "ANTHONY STARK"
    - rules [list of CorporateRule objects]: [EXPANDABLE] list of card spending rules.
    - tags [list of strings]: list of strings for tagging. ex: ["travel", "food"]
    - street_line_1 [string, default sub-issuer street line 1]: card holder main address. ex: "Av. Paulista, 200"
    - street_line_2 [string, default sub-issuer street line 2]: card holder address complement. ex: "Apto. 123"
    - district [string, default sub-issuer district]: card holder address district/neighbourhood. ex: "Bela Vista"
    - city [string, default sub-issuer city]: card holder address city. ex: "Rio de Janeiro"
    - state_code [string, default sub-issuer state code]: card holder address state. ex: "GO"
    - zip_code [string, default sub-issuer zip code]: card holder address zip code. ex: "01311-200"
    - type [string]: card type. ex: "virtual"
    - status [string]: current CorporateCard status. ex: "active", "blocked", "canceled", "expired"
    - number [string]: [EXPANDABLE] masked card number. Expand to unmask the value. ex: "123"
    - security_code [string]: [EXPANDABLE] masked card verification value (cvv). Expand to unmask the value. ex: "123"
    - expiration [datetime.datetime]: [EXPANDABLE] masked card expiration datetime. Expand to unmask the value. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the CorporateCard. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the CorporateCard. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, holder_id, id=None, holder_name=None, display_name=None, rules=None, tags=None, street_line_1=None,
                 street_line_2=None, district=None, city=None, state_code=None, zip_code=None, type=None,
                 status=None, number=None, security_code=None, expiration=None, updated=None, created=None
        ):
        Resource.__init__(self, id=id)

        self.holder_id = holder_id
        self.holder_name = holder_name
        self.display_name = display_name
        self.rules = parse_rules(rules)
        self.tags = tags
        self.street_line_1 = street_line_1
        self.street_line_2 = street_line_2
        self.district = district
        self.city = city
        self.state_code = state_code
        self.zip_code = zip_code
        self.type = type
        self.status = status
        self.number = number
        self.security_code = security_code
        self.expiration = check_datetime(expiration)
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": CorporateCard, "name": "CorporateCard"}


def create(card, expand=None, user=None):
    """# Create CorporateCard
    Send a CorporateCard object for creation at the Stark Bank API
    If the CorporateCard was not used in the last purchase, this resource will return it.
    ## Parameters (required):
    - card [CorporateCard object]: CorporateCard object to be created in the API.
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateCard object with updated attributes
    """
    path = "{endpoint}/{sub_resource}".format(endpoint=endpoint(_resource), sub_resource="token")
    json = rest.post_raw(path=path, payload=api_json(card), query={"expand": expand}, user=user).json()
    return from_api_json(_resource, json[last_name(_resource)])


def query(limit=None, after=None, before=None, status=None, types=None, holder_ids=None, ids=None, tags=None,
          expand=None, user=None):
    """# Retrieve CorporateCards
    Receive a generator of CorporateCard objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled", "expired"]
    - types [list of strings, default None]: card type. ex: ["virtual"]
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - generator of CorporateCard objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        types=types,
        holder_ids=holder_ids,
        ids=ids,
        tags=tags,
        expand=expand,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, types=None, holder_ids=None, ids=None,
         tags=None, expand=None, user=None):
    """# Retrieve paged CorporateCards
    Receive a list of up to 100 CorporateCard objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled", "expired"]
    - types [list of strings, default None]: card type. ex: ["virtual"]
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - list of CorporateCard objects with updated attributes
    - cursor to retrieve the next page of CorporateCard objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        types=types,
        holder_ids=holder_ids,
        ids=ids,
        tags=tags,
        expand=expand,
        user=user,
    )


def get(id, expand=None, user=None):
    """# Retrieve a specific CorporateCard
    Receive a single CorporateCard object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. ex: ["rules", "security_code", "number", "expiration"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateCard object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, expand=expand, user=user)


def update(id, status=None, display_name=None, rules=None, tags=None, pin=None, user=None):
    """# Update a CorporateCard entity
    Update a CorporateCard by passing id.
    ## Parameters (required):
    - id [string]: CorporateCard id. ex: '5656565656565656'
    ## Parameters (optional):
    - status [string, default None]: You may block the CorporateCard by passing 'blocked' or activate by passing 'active' in the status
    - display_name [string, default None]: card displayed name. ex: "ANTHONY EDWARD"
    - pin [string, default None]: You may unlock your physical card by passing its PIN. This is also the PIN you use to authorize a purhcase.
    - rules [list of CorporateRule objects, default None]: list of dictionaries with "amount": int, "currencyCode": string, "id": string, "interval": string, "name": string pairs.
    - tags [list of strings]: list of strings for tagging. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - target CorporateCard with updated attributes
    """
    payload = {
        "status": status,
        "display_name": display_name,
        "pin": pin,
        "rules": [api_json(rule) for rule in rules] if rules else None,
        "tags": tags,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel a CorporateCard entity
    Cancel a CorporateCard entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: CorporateCard unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - canceled CorporateCard object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ...utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_date, check_datetime
from ..__merchantcard import _resource as _merchant_card_resource


class Log(Resource):
    """# merchantcard.Log object
    Check out our API Documentation at https://starkbank.com/docs/api#merchant-card
    """

    def __init__(self, id, created, updated, type, errors, card):
        Resource.__init__(self, id=id)
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)
        self.type = type
        self.errors = errors
        self.card = from_api_json(_merchant_card_resource, card)


_resource = {"class": Log, "name": "MerchantCardLog"}


def get(id, user=None):
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, card_ids=None, after=None, before=None, user=None, types=None):
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        card_ids=card_ids,
        user=user,
        types=types,
    )


def page(cursor=None, limit=None, card_ids=None, after=None, before=None, user=None, types=None):
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        card_ids=card_ids,
        user=user,
    )




from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class SplitProfile(Resource):
    """# SplitProfile object
    When you create a Split, the entity SplitProfile will be automatically created, if you haven't create a Split yet, you can use the put method to create your SplitProfile.

    ## Parameters (optional):
    - interval [string]: frequency of transfer, default "week". Options: "day", "week", "month"
    - delay [DateInterval or integer]:  how long the amount will stay at the workspace in milliseconds, ex: 604800
    - tags [list of strings, default []]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when the splitProfile is created. ex: "5656565656565656"
    - status [string]: current splitProfile status. ex: "created"
    - created [datetime.datetime]: creation datetime for the splitProfile. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the splitProfile. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, delay, interval, tags=None, id=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.interval = interval
        self.delay = delay
        self.tags = tags
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": SplitProfile, "name": "SplitProfile"}


def put(splitProfile, user=None):
    """# Create SplitProfile or update it if you already have it created
    Send a list of SplitProfile objects for creation in the Stark Bank API
    ## Parameters (required):
    - splitProfile [list of SplitProfile objects]: list of SplitProfile objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of SplitProfile objects with updated attributes
    """
    return rest.put_multi(resource=_resource, entities=splitProfile, user=user)


def get(id, user=None):
    """# Retrieve a specific SplitProfile
    Receive a single SplitProfile object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - SplitProfile object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, transaction_ids=None, status=None, tax_id=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve SplitProfile
    Receive a generator of SplitProfile objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created or updated only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created or updated only before specified date. ex: datetime.date(2020, 3, 10)
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of SplitProfile objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        user=user,
    )


def page(cursor=None, after=None, before=None, tags=None, ids=None, receiver_ids=None, status=None, limit=None, user=None):
    """# Retrieve paged Split Profiles
    Receive a list of up to 100 Split Profiles objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - receiver_ids [list of strings, default None]: list of receiver ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Split objects with updated attributes
    - cursor to retrieve the next page of Split objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        receiver_ids=receiver_ids,
        status=status,
        user=user,
    )



from starkcore.utils.subresource import SubResource


class UtilityPreview(SubResource):

    """# UtilityPreview object
    A UtilityPreview is used to get information from a Utility Payment you received before confirming the payment.
    ## Attributes (return-only):
    - amount [int]: final amount to be paid. ex: 23456 (= R$ 234.56)
    - name [string]: beneficiary full name. ex: "Light Company"
    - description [string]: utility payment description. ex: "Utility Payment - Light Company"
    - line [string]: Number sequence that identifies the payment. ex: "82660000002 8 44361143007 7 41190025511 7 00010601813 8"
    - bar_code [string]: Bar code number that identifies the payment. ex: "82660000002443611430074119002551100010601813"
    """

    def __init__(self, amount, name, description, line, bar_code):
        self.amount = amount
        self.name = name
        self.description = description
        self.line = line
        self.bar_code = bar_code


_sub_resource = {"class": UtilityPreview, "name": "UtilityPreview"}



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class DarfPayment(Resource):
    """# DarfPayment object
    When you initialize a DarfPayment, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - description [string]: Text to be displayed in your statement (min. 10 characters). ex: "payment ABC"
    - revenue_code [string]: 4-digit tax code assigned by Federal Revenue. ex: "5948"
    - tax_id [string]: tax id (formatted or unformatted) of the payer. ex: "12.345.678/0001-95"
    - competence [datetime.date or string]: competence month of the service. ex: datetime.date(2021, 4, 30)
    - nominal_amount [int]: amount due in cents without fee or interest. ex: 23456 (= R$ 234.56)
    - fine_amount [int]: fixed amount due in cents for fines. ex: 234 (= R$ 2.34)
    - interest_amount [int]: amount due in cents for interest. ex: 456 (= R$ 4.56)
    - due [datetime.date or string]: due date for payment. ex: datetime.date(2021, 5, 17)
    ## Parameters (optional):
    - reference_number [string]: number assigned to the region of the tax. ex: "08.1.17.00-4"
    - scheduled [datetime.date or string, default today]: payment scheduled date. ex: datetime.date(2021, 5, 10)
    - tags [list of strings]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when payment is created. ex: "5656565656565656"
    - status [string]: current payment status. ex: "success" or "failed"
    - amount [int]: Total amount due calculated from other amounts. ex: 24146 (= R$ 241.46)
    - fee [integer]: fee charged when the DarfPayment is processed. ex: 0 (= R$ 0.00)
    - transaction_ids [list of strings]: ledger transaction ids linked to this DarfPayment. ex: ["19827356981273"]
    - updated [datetime.datetime]: update datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """
    def __init__(self, description, revenue_code, tax_id, competence, nominal_amount, fine_amount, interest_amount,
                 due, reference_number=None, scheduled=None, tags=None, id=None, status=None, amount=None, fee=None,
                 transaction_ids=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.revenue_code = revenue_code
        self.tax_id = tax_id
        self.competence = check_date(competence)
        self.reference_number = reference_number
        self.fine_amount = fine_amount
        self.interest_amount = interest_amount
        self.due = check_date(due)
        self.description = description
        self.tags = tags
        self.scheduled = check_date(scheduled)
        self.status = status
        self.amount = amount
        self.nominal_amount = nominal_amount
        self.fee = fee
        self.transaction_ids = transaction_ids
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": DarfPayment, "name": "DarfPayment"}


def create(payments, user=None):
    """# Create DarfPayments
    Send a list of DarfPayment objects for creation in the Stark Bank API
    ## Parameters (required):
    - payments [list of DarfPayment objects]: list of DarfPayment objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of DarfPayment objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=payments, user=user)


def get(id, user=None):
    """# Retrieve a specific DarfPayment
    Receive a single DarfPayment object previously created by the Stark Bank API by passing its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - DarfPayment object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific DarfPayment pdf file
    Receive a single DarfPayment pdf file generated in the Stark Bank API by passing its id.
    Only valid for darf payments with "success" status.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - DarfPayment pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def query(limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve DarfPayments
    Receive a generator of DarfPayment objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Project object, default None]: Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of DarfPayment objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve paged DarfPayments
    Receive a list of up to 100 DarfPayment objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of DarfPayment objects with updated attributes
    - cursor to retrieve the next page of DarfPayment objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def delete(id, user=None):
    """# Delete a DarfPayment entity
    Delete a DarfPayment entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: DarfPayment unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deleted DarfPayment object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ...utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..__taxpayment import _resource as _payment_resource


class Log(Resource):
    """# taxpayment.Log object
    Every time a TaxPayment entity is modified, a corresponding taxpayment.Log
    is generated for the entity. This log is never generated by the user, but it can
    be retrieved to check additional information on the TaxPayment.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - payment [TaxPayment]: TaxPayment entity to which the log refers to.
    - errors [list of strings]: list of errors linked to this TaxPayment event.
    - type [string]: type of the TaxPayment event which triggered the log creation. ex: "processing" or "success"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, created, type, errors, payment):
        Resource.__init__(self, id=id)

        self.created = check_datetime(created)
        self.type = type
        self.errors = errors
        self.payment = from_api_json(_payment_resource, payment)


_resource = {"class": Log, "name": "TaxPaymentLog"}


def get(id, user=None):
    """# Retrieve a specific taxpayment.Log
    Receive a single taxpayment.Log object previously created by the Stark Bank API by passing its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - taxpayment.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, types=None, payment_ids=None, user=None):
    """# Retrieve taxpayment.Log's
    Receive a generator of taxpayment.Log objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by event types. ex: "processing" or "success"
    - payment_ids [list of strings, default None]: list of TaxPayment ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Project object, default None]: Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of taxpayment.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        payment_ids=payment_ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, types=None, payment_ids=None, user=None):
    """# Retrieve paged taxpayment.Logs
    Receive a list of up to 100 taxpayment.Log objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: "success" or "failed"
    - payment_ids [list of strings, default None]: list of TaxPayment ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of taxpayment.Log objects with updated attributes
    - cursor to retrieve the next page of taxpayment.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        payment_ids=payment_ids,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class BoletoHolmes(Resource):
    """# BoletoHolmes object
    When you initialize a BoletoHolmes, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - boleto_id [string]: investigated boleto entity ID. ex: "5656565656565656"
    ## Parameters (optional):
    - tags [list of strings]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when holmes is created. ex: "5656565656565656"
    - status [string]: current holmes status. ex: "solving" or "solved"
    - result [string]: result of boleto status investigation. ex: "paid" or "cancelled"
    - created [datetime.datetime]: creation datetime for the holmes. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the holmes. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, boleto_id, tags=None, status=None, result=None, created=None, updated=None, id=None):
        Resource.__init__(self, id=id)

        self.boleto_id = boleto_id
        self.tags = tags
        self.status = status
        self.result = result
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": BoletoHolmes, "name": "BoletoHolmes"}


def create(holmes, user=None):
    """# Create BoletoHolmes
    Send a list of BoletoHolmes objects for creation in the Stark Bank API
    ## Parameters (required):
    - holmes [list of BoletoHolmes objects]: list of BoletoHolmes objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of BoletoHolmes objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=holmes, user=user)


def get(id, user=None):
    """# Retrieve a specific BoletoHolmes
    Receive a single BoletoHolmes object previously created by the Stark Bank API by passing its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - BoletoHolmes object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, tags=None, ids=None, status=None, boleto_id=None, user=None):
    """# Retrieve BoletoHolmes
    Receive a generator of BoletoHolmes objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "solved"
    - boleto_id [string, default None]: filter for holmes that investigate a specific boleto by its ID. ex: "5656565656565656"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of BoletoHolmes objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        boleto_id=boleto_id,
        user=user,
    )


def page(cursor=None, limit=None, user=None):
    """# Retrieve paged BoletoHolmes
    Receive a list of up to 100 BoletoHolmes objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "solved"
    - boleto_id [string, default None]: filter for holmes that investigate a specific boleto by its ID. ex: "5656565656565656"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of BoletoHolmes objects with updated attributes
    - cursor to retrieve the next page of BoletoHolmes objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        user=user,
    )



from starkcore.utils.subresource import SubResource


class Payment(SubResource):
    """# Invoice.Payment object
    When an Invoice is paid, its Payment sub-resource will become available.
    It carries all the available information about the invoice payment.
    ## Attributes (return-only):
    - amount [integer]: amount in cents that was paid. ex: 1234 (= R$ 12.34)
    - name [string]: payer full name. ex: "Anthony Edward Stark"
    - tax_id [string]: payer tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    - bank_code [string]: code of the payer bank institution in Brazil. ex: "20018183"
    - branch_code [string]: payer bank account branch. ex: "1357-9"
    - account_number [string]: payer bank account number. ex: "876543-2"
    - account_type [string]: payer bank account type. ex: "checking", "savings", "salary" or "payment"
    - end_to_end_id [string]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - method [string]: payment method that was used. ex: "pix"
    """

    def __init__(self, name, tax_id, bank_code, branch_code, account_number, account_type, amount, end_to_end_id, method):
        self.name = name
        self.tax_id = tax_id
        self.bank_code = bank_code
        self.branch_code = branch_code
        self.account_number = account_number
        self.account_type = account_type
        self.amount = amount
        self.end_to_end_id = end_to_end_id
        self.method = method


_sub_resource = {"class": Payment, "name": "Payment"}



from starkcore.utils.subresource import SubResource


class Rule(SubResource):

    def __init__(self, key, value):
        self.key = key
        self.value = value


_sub_resource = {"class": Rule, "name": "Rule"}



from ..utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_date
from starkcore.utils.resource import Resource
from .__taxpreview import _sub_resource as _tax_preview_sub_resource
from .__brcodepreview import _sub_resource as _brcode_preview_sub_resource
from .__boletopreview import _sub_resource as _boleto_preview_sub_resource
from .__utilitypreview import _sub_resource as _utility_preview_sub_resource


_sub_resource_by_type = {
    "brcode-payment": _brcode_preview_sub_resource,
    "boleto-payment": _boleto_preview_sub_resource,
    "utility-payment": _utility_preview_sub_resource,
    "tax-payment": _tax_preview_sub_resource,
}


class PaymentPreview(Resource):

    """# PaymentPreview object
    A PaymentPreview is used to get information from a payment code before confirming the payment.
    This resource can be used to preview BR Codes and bar codes of boleto, tax and utility payments
    ## Parameters (required):
    - id [string]: Main identification of the payment. This should be the BR Code for Pix payments and lines or bar codes for payment slips. ex: "34191.09008 63571.277308 71444.640008 5 81960000000062", "00020126580014br.gov.bcb.pix0136a629532e-7693-4846-852d-1bbff817b5a8520400005303986540510.005802BR5908T'Challa6009Sao Paulo62090505123456304B14A"
    ## Parameters (optional):
    - scheduled [datetime.date or string, default today]: intended payment date. Right now, this parameter only has effect on BrcodePreviews. ex: datetime.date(2020, 3, 10)
    ## Attributes (return-only):
    - type [string]: Payment type. ex: "brcode-payment", "boleto-payment", "utility-payment" or "tax-payment"
    - payment [BrcodePreview, BoletoPreview, UtilityPreview or TaxPreview]: Information preview of the informed payment.
    """

    def __init__(self, id, scheduled=None, type=None, payment=None):
        Resource.__init__(self, id=id)
        self.scheduled = check_date(scheduled)
        self.type = type
        self.payment = payment
        if type in _sub_resource_by_type:
            self.payment = from_api_json(resource=_sub_resource_by_type[type], json=payment)


_resource = {"class": PaymentPreview, "name": "PaymentPreview"}


def create(previews, user=None):
    """# Create PaymentPreviews
    Send a list of PaymentPreviews objects for processing in the Stark Bank API
    ## Parameters (required):
    - previews [list of PaymentPreviews objects]: list of PaymentPreviews objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of PaymentPreviews objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=previews, user=user)



from starkcore.utils.resource import Resource
from ..utils import rest
from starkcore.utils.api import from_api_json
from .allowedinstallment.__allowedinstallment import AllowedInstallment
from .allowedinstallment.__allowedinstallment import _sub_resource as _allowed_installments_sub_resource
from starkcore.utils.checks import check_date, check_datetime
from .__purchase import _resource as purchase_resource


class MerchantSession(Resource):
    """# MerchantSession object
    Check out our API Documentation at https://starkbank.com/docs/api#merchant-session
    """

    def __init__(self, allowed_funding_types, allowed_installments, expiration, id=None, allowed_ips=None,
                 challenge_mode=None, created=None, status=None, tags=None, updated=None, uuid=None):
        Resource.__init__(self, id=id)

        self.allowed_funding_types = allowed_funding_types
        self.allowed_installments = _parse_allowed_installments(allowed_installments)
        self.allowed_ips = allowed_ips
        self.challenge_mode = challenge_mode
        self.expiration = expiration
        self.status = status
        self.tags = tags
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)
        self.uuid = uuid


_resource = {"class": MerchantSession, "name": "MerchantSession"}


def _parse_allowed_installments(allowed_installments):
    if allowed_installments is None:
        return []
    parsed_allowed_installments = []
    for allowed_installment in allowed_installments:
        if isinstance(allowed_installment, AllowedInstallment):
            parsed_allowed_installments.append(allowed_installment)
            continue
        parsed_allowed_installments.append(from_api_json(_allowed_installments_sub_resource, allowed_installment))
    return parsed_allowed_installments


def create(merchant_session, user=None):
    return rest.post_single(resource=_resource, entity=merchant_session, user=user)


def get(id, user=None):
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def purchase(uuid, purchase, user=None):
    return rest.post_sub_resource(resource=_resource, id=uuid, sub_resource=purchase_resource, entity=purchase, user=user)




from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_datetime_or_date, check_date


class InvoicePullRequest(Resource):
    """# InvoicePullRequest object
    When you initialize an InvoicePullRequest, the entity will not be automatically
    sent to the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - subscription_id [string]: Unique of the InvoicePullSubscription related to the invoice. ex: "5656565656565656"
    - invoice_id [string]: Id of the invoice previously created to be sent for payment. ex: "5656565656565656"
    - due [datetime.datetime or string]: payment scheduled date in UTC ISO format. ex: "2023-10-28T17:59:26.249976+00:00"
    ## Parameters (optional):
    - attempt_type [string, default "default"]: attempt type for the payment. Options: "default", "retry".
    - tags [list of strings, default []]: list of strings for tagging
    - external_id [string, default None]: a string that must be unique among all your InvoicePullRequests. Duplicated external_ids will cause failures. ex: "my-external-id"
    - display_description [string, default None]: Description to be shown to the payer. ex: "Payment for services"
    ## Attributes (return-only):
    - id [string]: unique id returned when InvoicePullRequest is created. ex: "5656565656565656"
    - status [string]: current InvoicePullRequest status. ex: "pending", "scheduled", "success", "failed", "canceled"
    - installment_id [string]: unique id of the installment related to this request. ex: "5656565656565656"
    - created [datetime.datetime]: creation datetime for the InvoicePullRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the InvoicePullRequest. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, subscription_id, invoice_id, due, attempt_type=None, tags=None, external_id=None,
                 display_description=None, id=None, status=None, installment_id=None,
                 created=None, updated=None):
        Resource.__init__(self, id=id)

        self.subscription_id = subscription_id
        self.invoice_id = invoice_id
        self.due = check_datetime_or_date(due)
        self.attempt_type = attempt_type
        self.tags = tags
        self.external_id = external_id
        self.display_description = display_description
        self.status = status
        self.installment_id = installment_id
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": InvoicePullRequest, "name": "InvoicePullRequest"}


def create(requests, user=None):
    """# Create InvoicePullRequests
    Send a list of InvoicePullRequest objects for creation in the Stark Bank API
    ## Parameters (required):
    - requests [list of InvoicePullRequest objects]: list of InvoicePullRequest objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of InvoicePullRequest objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=requests, user=user)


def get(id, user=None):
    """# Retrieve a specific InvoicePullRequest
    Receive a single InvoicePullRequest object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - InvoicePullRequest object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, invoice_ids=None, subscription_ids=None, external_ids=None, 
          tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve InvoicePullRequests
    Receive a generator of InvoicePullRequest objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["success", "failed"]
    - invoice_ids [list of strings, default None]: list of Invoice ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - subscription_ids [list of strings, default None]: list of InvoicePullSubscription ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - external_ids [list of strings, default None]: list of external_ids to filter retrieved objects. ex: ["my-external-id-1", "my-external-id-2"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of InvoicePullRequest objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        invoice_ids=invoice_ids,
        subscription_ids=subscription_ids,
        external_ids=external_ids,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, invoice_ids=None, subscription_ids=None, external_ids=None,
         tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged InvoicePullRequests
    Receive a list of up to 100 InvoicePullRequest objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["success", "failed"]
    - invoice_ids [list of strings, default None]: list of Invoice ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - subscription_ids [list of strings, default None]: list of InvoicePullSubscription ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - external_ids [list of strings, default None]: list of external_ids to filter retrieved objects. ex: ["my-external-id-1", "my-external-id-2"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of InvoicePullRequest objects with updated attributes
    - cursor to retrieve the next page of InvoicePullRequest objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        invoice_ids=invoice_ids,
        subscription_ids=subscription_ids,
        external_ids=external_ids,
        tags=tags,
        ids=ids,
        user=user,
    )


def cancel(id, user=None):
    """# Cancel an InvoicePullRequest entity
    Cancel an InvoicePullRequest entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: InvoicePullRequest unique id. ex: '5656565656565656'
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - canceled InvoicePullRequest object
    """
    return rest.delete_id(resource=_resource, id=id, user=user) 


from starkcore.utils.subresource import SubResource


class TaxPreview(SubResource):

    """# TaxPreview object
    A TaxPreview is used to get information from a Tax Payment you received before confirming the payment.
    ## Attributes (return-only):
    - amount [int]: final amount to be paid. ex: 23456 (= R$ 234.56)
    - name [string]: beneficiary full name. ex: "Iron Throne"
    - description [string]: tax payment description. ex: "ISS Payment - Iron Throne"
    - line [string]: Number sequence that identifies the payment. ex: "85660000006 6 67940064007 5 41190025511 7 00010601813 8"
    - bar_code [string]: Bar code number that identifies the payment. ex: "85660000006679400640074119002551100010601813"
    """

    def __init__(self, amount, name, description, line, bar_code):
        self.amount = amount
        self.name = name
        self.description = description
        self.line = line
        self.bar_code = bar_code


_sub_resource = {"class": TaxPreview, "name": "TaxPreview"}



from ..utils import rest
from .rule.__rule import Rule
from .rule.__rule import _sub_resource as _rule_resource
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class BrcodePayment(Resource):

    """# BrcodePayment object
    When you initialize a BrcodePayment, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - brcode [string]: String loaded directly from the QR Code or copied from the invoice. ex: "00020126580014br.gov.bcb.pix0136a629532e-7693-4846-852d-1bbff817b5a8520400005303986540510.005802BR5908T'Challa6009Sao Paulo62090505123456304B14A"
    - tax_id [string]: receiver tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - description [string]: Text to be displayed in your statement (min. 10 characters). ex: "payment ABC"
    ## Parameters (conditionally required):
    - amount [int, default None]: If the BRCode does not provide an amount, this parameter is mandatory, else it is optional. ex: 23456 (= R$ 234.56)
    ## Parameters (optional):
    - scheduled [datetime.date, datetime.datetime or string, default now]: payment scheduled date or datetime. ex: datetime.datetime(2020, 3, 10, 15, 17, 3)
    - tags [list of strings, default []]: list of strings for tagging
    - rules [list of BrcodePayment.Rule, default []]: list of BrcodePayment.Rule objects for modifying payment behavior. ex: [Rule(key="resendingLimit", value=5)]
    ## Attributes (return-only):
    - id [string]: unique id returned when payment is created. ex: "5656565656565656"
    - name [string]: receiver name. ex: "Jon Snow"
    - status [string]: current payment status. ex: "success" or "failed"
    - type [string]: brcode type. ex: "static" or "dynamic"
    - transaction_ids [list of strings]: ledger transaction ids linked to this payment. ex: ["19827356981273"]
    - fee [integer]: fee charged by this brcode payment. ex: 50 (= R$ 0.50)
    - updated [datetime.datetime]: latest update datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, brcode, tax_id, description, amount=None, scheduled=None, tags=None, rules=None, id=None, name=None,
                 status=None, type=None, transaction_ids=None, fee=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.brcode = brcode
        self.tax_id = tax_id
        self.description = description
        self.amount = amount
        self.scheduled = check_date(scheduled)
        self.tags = tags
        self.rules = _parse_rules(rules)
        self.name = name
        self.status = status
        self.type = type
        self.transaction_ids = transaction_ids
        self.fee = fee
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": BrcodePayment, "name": "BrcodePayment"}


def _parse_rules(rules):
    if rules is None:
        return None
    parsed_rules = []
    for rule in rules:
        if isinstance(rule, Rule):
            parsed_rules.append(rule)
            continue
        parsed_rules.append(from_api_json(_rule_resource, rule))
    return parsed_rules


def create(payments, user=None):
    """# Create BrcodePayments
    Send a list of BrcodePayment objects for creation in the Stark Bank API
    ## Parameters (required):
    - payments [list of BrcodePayment objects]: list of BrcodePayment objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of BrcodePayment objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=payments, user=user)


def get(id, user=None):
    """# Retrieve a specific BrcodePayment
    Receive a single BrcodePayment object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - BrcodePayment object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific BrcodePayment pdf file
    Receive a single BrcodePayment pdf receipt file generated in the Stark Bank API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - BrcodePayment pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def query(limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve BrcodePayments
    Receive a generator of BrcodePayment objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of BrcodePayment objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def page(cursor=None, after=None, before=None, tags=None, ids=None, status=None, limit=None, user=None):
    """# Retrieve paged BrcodePayments
    Receive a list of up to 100 BrcodePayment objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of BrcodePayment objects with updated attributes
    - cursor to retrieve the next page of BrcodePayment objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def update(id, status=None, user=None):
    """# Update BrcodePayment entity
    Update a BrcodePayment by passing its Id, if it hasn't been paid yet.
    ## Parameters (required):
    - id [string]: BrcodePayment id. ex: '5656565656565656'
    ## Parameters (required):
    - status [string]: You may cancel the payment by passing 'canceled' in the status
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - target BrcodePayment with updated attributes
    """
    payload = {
        "status": status
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)



from starkcore.utils.subresource import SubResource


class Rule(SubResource):
    """# Transfer.Rule object
    The Transfer.Rule object modifies the behavior of Transfer objects when passed as an argument upon their creation.
    ## Parameters (required):
    - key [string]: Rule to be customized, describes what Transfer behavior will be altered. ex: "resendingLimit"
    - value [integer]: Value of the rule. ex: 5
    """

    def __init__(self, key, value):
        self.key = key
        self.value = value


_sub_resource = {"class": Rule, "name": "Rule"}



from ...utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..__invoice import _resource as _invoice_resource


class Log(Resource):
    """# invoice.Log object
    Every time an Invoice entity is updated, a corresponding invoice.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information on the Invoice.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - invoice [Invoice]: Invoice entity to which the log refers to.
    - errors [list of strings]: list of errors linked to this Invoice event
    - type [string]: type of the Invoice event which triggered the log creation. ex: "registered" or "paid"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, created, type, errors, invoice):
        Resource.__init__(self, id=id)

        self.created = check_datetime(created)
        self.type = type
        self.errors = errors
        self.invoice = from_api_json(_invoice_resource, invoice)


_resource = {"class": Log, "name": "InvoiceLog"}


def get(id, user=None):
    """# Retrieve a specific invoice.Log
    Receive a single invoice.Log object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - invoice.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, types=None, invoice_ids=None, user=None):
    """# Retrieve invoice.Logs
    Receive a generator of invoice.Log objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: "paid" or "registered"
    - invoice_ids [list of strings, default None]: list of Invoice ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of invoice.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        invoice_ids=invoice_ids,
        user=user,
    )


def page(cursor=None, after=None, before=None, types=None, invoice_ids=None, limit=None, user=None):
    """# Retrieve paged invoice.Logs
    Receive a list of up to 100 invoice.Log objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: "paid" or "registered"
    - invoice_ids [list of strings, default None]: list of Invoice ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of invoice.Log objects with updated attributes
    - cursor to retrieve the next page of invoice.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        invoice_ids=invoice_ids,
        user=user,
    )


def pdf(id, user=None):
    """# Retrieve a reversed Invoice.Log pdf file
    Receive a reversed Invoice.Log pdf receipt file generated in the Stark Bank API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Invoice pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")



from starkcore.utils.resource import Resource
from ..utils import rest
from starkcore.utils.checks import check_date, check_datetime


class MerchantPurchase(Resource):
    """# MerchantPurchase object
    Check out our API Documentation at https://starkbank.com/docs/api#merchant-purchase
    """

    def __init__(self, amount, card_id, funding_type, installment_count, id=None, card_expiration=None,
                 card_number=None, card_security_code=None,holder_name=None, holder_email=None, holder_phone=None,
                 billing_country_code=None, billing_city=None,billing_state_code=None, billing_street_line_1=None,
                 billing_street_line_2=None, billing_zip_code=None, metadata=None, card_ending=None,
                 challenge_mode=None, challenge_url=None, created=None, currency_code=None, end_to_end_id=None,
                 fee=None, network=None, source=None, status=None, tags=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.installment_count = installment_count
        self.card_expiration = card_expiration
        self.card_number = card_number
        self.card_security_code = card_security_code
        self.holder_name = holder_name
        self.holder_email = holder_email
        self.holder_phone = holder_phone
        self.funding_type = funding_type
        self.billing_country_code = billing_country_code
        self.billing_city = billing_city
        self.billing_state_code = billing_state_code
        self.billing_street_line_1 = billing_street_line_1
        self.billing_street_line_2 = billing_street_line_2
        self.billing_zip_code = billing_zip_code
        self.metadata = metadata
        self.card_ending = card_ending
        self.card_id = card_id
        self.challenge_mode = challenge_mode
        self.challenge_url = challenge_url
        self.currency_code = currency_code
        self.end_to_end_id = end_to_end_id
        self.fee = fee
        self.network = network
        self.source = source
        self.status = status
        self.tags = tags
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": MerchantPurchase, "name": "MerchantPurchase"}


def create(merchant_purchase, user=None):
    return rest.post_single(resource=_resource, entity=merchant_purchase, user=user)


def get(id, user=None):
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, tags=None, ids=None, user=None):
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, ids=None, user=None):
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def update(id, status=None, amount=None, user=None):
    payload = {
        "status": status,
        "amount": amount
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)

